@@ -1,6 +1,7 @@
 import socket
 
 from mock import ANY, MagicMock, patch
+from operator import itemgetter
 import six
 from . import unittest
 
@@ -27,6 +28,7 @@ def mock_conn(conn, success=True):
     else:
         mocked.send.return_value = Future().failure(Exception())
     conn.return_value = mocked
+    conn.recv.return_value = []
 
 
 class TestSimpleClient(unittest.TestCase):
@@ -93,7 +95,7 @@ class TestSimpleClient(unittest.TestCase):
         mock_conn(mocked_conns[('kafka03', 9092)], success=False)
         future = Future()
         mocked_conns[('kafka02', 9092)].send.return_value = future
-        mocked_conns[('kafka02', 9092)].recv.side_effect = lambda: future.success('valid response')
+        mocked_conns[('kafka02', 9092)].recv.return_value = [('valid response', future)]
 
         def mock_get_conn(host, port, afi):
             return mocked_conns[(host, port)]
@@ -117,9 +119,10 @@ class TestSimpleClient(unittest.TestCase):
         mock_conn(conn)
 
         brokers = [
-            BrokerMetadata(0, 'broker_1', 4567),
-            BrokerMetadata(1, 'broker_2', 5678)
+            BrokerMetadata(0, 'broker_1', 4567, None),
+            BrokerMetadata(1, 'broker_2', 5678, None)
         ]
+        resp0_brokers = list(map(itemgetter(0, 1, 2), brokers))
 
         topics = [
             (NO_ERROR, 'topic_1', [
@@ -137,7 +140,7 @@ class TestSimpleClient(unittest.TestCase):
                 (NO_ERROR, 2, 0, [0, 1], [0, 1])
             ])
         ]
-        protocol.decode_metadata_response.return_value = MetadataResponse[0](brokers, topics)
+        protocol.decode_metadata_response.return_value = MetadataResponse[0](resp0_brokers, topics)
 
         # client loads metadata at init
         client = SimpleClient(hosts=['broker_1:4567'])
@@ -167,9 +170,10 @@ class TestSimpleClient(unittest.TestCase):
         mock_conn(conn)
 
         brokers = [
-            BrokerMetadata(0, 'broker_1', 4567),
-            BrokerMetadata(1, 'broker_2', 5678)
+            BrokerMetadata(0, 'broker_1', 4567, None),
+            BrokerMetadata(1, 'broker_2', 5678, None)
         ]
+        resp0_brokers = list(map(itemgetter(0, 1, 2), brokers))
 
         topics = [
             (NO_LEADER, 'topic_still_creating', []),
@@ -179,7 +183,7 @@ class TestSimpleClient(unittest.TestCase):
                 (NO_LEADER, 1, -1, [], []),
             ]),
         ]
-        protocol.decode_metadata_response.return_value = MetadataResponse[0](brokers, topics)
+        protocol.decode_metadata_response.return_value = MetadataResponse[0](resp0_brokers, topics)
 
         client = SimpleClient(hosts=['broker_1:4567'])
 
@@ -197,9 +201,10 @@ class TestSimpleClient(unittest.TestCase):
         mock_conn(conn)
 
         brokers = [
-            BrokerMetadata(0, 'broker_1', 4567),
-            BrokerMetadata(1, 'broker_2', 5678)
+            BrokerMetadata(0, 'broker_1', 4567, None),
+            BrokerMetadata(1, 'broker_2', 5678, None)
         ]
+        resp0_brokers = list(map(itemgetter(0, 1, 2), brokers))
 
         topics = [
             (NO_LEADER, 'topic_still_creating', []),
@@ -209,7 +214,7 @@ class TestSimpleClient(unittest.TestCase):
                 (NO_LEADER, 1, -1, [], []),
             ]),
         ]
-        decode_metadata_response.return_value = MetadataResponse[0](brokers, topics)
+        decode_metadata_response.return_value = MetadataResponse[0](resp0_brokers, topics)
 
         client = SimpleClient(hosts=['broker_1:4567'])
 
@@ -230,14 +235,15 @@ class TestSimpleClient(unittest.TestCase):
         mock_conn(conn)
 
         brokers = [
-            BrokerMetadata(0, 'broker_1', 4567),
-            BrokerMetadata(1, 'broker_2', 5678)
+            BrokerMetadata(0, 'broker_1', 4567, None),
+            BrokerMetadata(1, 'broker_2', 5678, None)
         ]
+        resp0_brokers = list(map(itemgetter(0, 1, 2), brokers))
 
         topics = [
             (NO_LEADER, 'topic_no_partitions', [])
         ]
-        protocol.decode_metadata_response.return_value = MetadataResponse[0](brokers, topics)
+        protocol.decode_metadata_response.return_value = MetadataResponse[0](resp0_brokers, topics)
 
         client = SimpleClient(hosts=['broker_1:4567'])
 
@@ -249,7 +255,7 @@ class TestSimpleClient(unittest.TestCase):
                 (NO_ERROR, 0, 0, [0, 1], [0, 1])
             ])
         ]
-        protocol.decode_metadata_response.return_value = MetadataResponse[0](brokers, topics)
+        protocol.decode_metadata_response.return_value = MetadataResponse[0](resp0_brokers, topics)
 
         # calling _get_leader_for_partition (from any broker aware request)
         # will try loading metadata again for the same topic
@@ -267,15 +273,16 @@ class TestSimpleClient(unittest.TestCase):
         mock_conn(conn)
 
         brokers = [
-            BrokerMetadata(0, 'broker_1', 4567),
-            BrokerMetadata(1, 'broker_2', 5678)
+            BrokerMetadata(0, 'broker_1', 4567, None),
+            BrokerMetadata(1, 'broker_2', 5678, None)
         ]
+        resp0_brokers = list(map(itemgetter(0, 1, 2), brokers))
 
         topics = [
             (NO_LEADER, 'topic_no_partitions', []),
             (UNKNOWN_TOPIC_OR_PARTITION, 'topic_unknown', []),
         ]
-        protocol.decode_metadata_response.return_value = MetadataResponse[0](brokers, topics)
+        protocol.decode_metadata_response.return_value = MetadataResponse[0](resp0_brokers, topics)
 
         client = SimpleClient(hosts=['broker_1:4567'])
 
@@ -294,9 +301,10 @@ class TestSimpleClient(unittest.TestCase):
         mock_conn(conn)
 
         brokers = [
-            BrokerMetadata(0, 'broker_1', 4567),
-            BrokerMetadata(1, 'broker_2', 5678)
+            BrokerMetadata(0, 'broker_1', 4567, None),
+            BrokerMetadata(1, 'broker_2', 5678, None)
         ]
+        resp0_brokers = list(map(itemgetter(0, 1, 2), brokers))
 
         topics = [
             (NO_ERROR, 'topic_noleader', [
@@ -304,7 +312,7 @@ class TestSimpleClient(unittest.TestCase):
                 (NO_LEADER, 1, -1, [], []),
             ]),
         ]
-        protocol.decode_metadata_response.return_value = MetadataResponse[0](brokers, topics)
+        protocol.decode_metadata_response.return_value = MetadataResponse[0](resp0_brokers, topics)
 
         client = SimpleClient(hosts=['broker_1:4567'])
         self.assertDictEqual(
@@ -330,7 +338,7 @@ class TestSimpleClient(unittest.TestCase):
                 (NO_ERROR, 1, 1, [1, 0], [1, 0])
             ]),
         ]
-        protocol.decode_metadata_response.return_value = MetadataResponse[0](brokers, topics)
+        protocol.decode_metadata_response.return_value = MetadataResponse[0](resp0_brokers, topics)
         self.assertEqual(brokers[0], client._get_leader_for_partition('topic_noleader', 0))
         self.assertEqual(brokers[1], client._get_leader_for_partition('topic_noleader', 1))
 
@@ -340,9 +348,10 @@ class TestSimpleClient(unittest.TestCase):
         mock_conn(conn)
 
         brokers = [
-            BrokerMetadata(0, 'broker_1', 4567),
-            BrokerMetadata(1, 'broker_2', 5678)
+            BrokerMetadata(0, 'broker_1', 4567, None),
+            BrokerMetadata(1, 'broker_2', 5678, None)
         ]
+        resp0_brokers = list(map(itemgetter(0, 1, 2), brokers))
 
         topics = [
             (NO_ERROR, 'topic_noleader', [
@@ -350,7 +359,7 @@ class TestSimpleClient(unittest.TestCase):
                 (NO_LEADER, 1, -1, [], []),
             ]),
         ]
-        protocol.decode_metadata_response.return_value = MetadataResponse[0](brokers, topics)
+        protocol.decode_metadata_response.return_value = MetadataResponse[0](resp0_brokers, topics)
 
         client = SimpleClient(hosts=['broker_1:4567'])
 
@@ -368,14 +377,15 @@ class TestSimpleClient(unittest.TestCase):
         mock_conn(conn)
 
         brokers = [
-            BrokerMetadata(0, 'broker_1', 4567),
-            BrokerMetadata(1, 'broker_2', 5678)
+            BrokerMetadata(0, 'broker_1', 4567, None),
+            BrokerMetadata(1, 'broker_2', 5678, None)
         ]
+        resp0_brokers = list(map(itemgetter(0, 1, 2), brokers))
 
         topics = [
             (UNKNOWN_TOPIC_OR_PARTITION, 'topic_doesnt_exist', []),
         ]
-        protocol.decode_metadata_response.return_value = MetadataResponse[0](brokers, topics)
+        protocol.decode_metadata_response.return_value = MetadataResponse[0](resp0_brokers, topics)
 
         client = SimpleClient(hosts=['broker_1:4567'])
 
@@ -389,7 +399,7 @@ class TestSimpleClient(unittest.TestCase):
     def test_correlation_rollover(self):
         with patch.object(SimpleClient, 'load_metadata_for_topics'):
             big_num = 2**31 - 3
-            client = SimpleClient(hosts=[], correlation_id=big_num)
+            client = SimpleClient(hosts=(), correlation_id=big_num)
             self.assertEqual(big_num + 1, client._next_id())
             self.assertEqual(big_num + 2, client._next_id())
             self.assertEqual(0, client._next_id())
